/*----------------------------------------------------------------------------
 Copyright:      
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        20.01.2013
 Description:  

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.   
------------------------------------------------------------------------------*/
#define F_CPU 12000000
#define DMX_BAUD 250000
#define DMX_BAUD_BREAK 80000

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>
#include "rf22.h"

volatile unsigned char dmx_buffer[514];
volatile unsigned char dmx_tmp_buffer[33];
volatile unsigned char new_dmx_data = 0;
volatile unsigned char dmx_buffer_pwm[6];
 
#define TRANSMIT_BLOCKS 16	//1 Block 32Channels max.16 Blocks 512Channels
#define TRANSMIT_REPEAT	64

#define LEDA_R_PIN_OUT DDRD |= (1<<PD7);
#define LEDA_G_PIN_OUT DDRB |= (1<<PB0);
#define LEDA_B_PIN_OUT DDRB |= (1<<PB1);

#define LEDB_R_PIN_OUT DDRD |= (1<<PD4);
#define LEDB_G_PIN_OUT DDRD |= (1<<PD5);
#define LEDB_B_PIN_OUT DDRD |= (1<<PD6);

#define LEDA_R_ON		PORTD |= (1<<PD7);
#define LEDA_R_OFF		PORTD &= ~(1<<PD7);
#define LEDA_B_ON		PORTB |= (1<<PB0);
#define LEDA_B_OFF		PORTB &= ~(1<<PB0);
#define LEDA_G_ON		PORTB |= (1<<PB1);
#define LEDA_G_OFF		PORTB &= ~(1<<PB1);

#define LEDB_R_ON		PORTD |= (1<<PD4);
#define LEDB_R_OFF		PORTD &= ~(1<<PD4);
#define LEDB_B_ON		PORTD |= (1<<PD5);
#define LEDB_B_OFF		PORTD &= ~(1<<PD5);
#define LEDB_G_ON		PORTD |= (1<<PD6);
#define LEDB_G_OFF		PORTD &= ~(1<<PD6);

//############################################################################
//DMX Senderoutine
ISR (USART_TX_vect)
//############################################################################
{
	static unsigned int  dmx_channel_tx_count = 1;
	static unsigned char dmx_tx_state = 0;
	
	switch (dmx_tx_state)
	{
		case (0):
			UBRR0   = (F_CPU / (DMX_BAUD_BREAK * 16L) - 1);
			UDR0 = 0; //RESET Frame
			dmx_tx_state = 1;
			break;

		case (1):
			UBRR0   = (F_CPU / (DMX_BAUD * 16L) - 1);
			UDR0 = 0; //Start Byte
			dmx_tx_state = 2;
			break;

		case (2):
			_delay_us(10);
			UDR0 = dmx_buffer[dmx_channel_tx_count];
			dmx_channel_tx_count++;
			
			if(dmx_channel_tx_count == 512)
			{
				dmx_channel_tx_count = 1;
				dmx_tx_state = 0;
			}
			break;
	}
}

//############################################################################
//Hier wird die Zeit gezhlt (Tick 1ms)
ISR (TIMER2_COMPA_vect)
//############################################################################
{
	static unsigned char pwm_counter = 0;
	if(pwm_counter == 255)
	{
		pwm_counter = 0;
		if(dmx_buffer_pwm[0] > 0){LEDA_R_ON}else{LEDA_R_OFF};
		if(dmx_buffer_pwm[1] > 0){LEDA_G_ON}else{LEDA_G_OFF};
		if(dmx_buffer_pwm[2] > 0){LEDA_B_ON}else{LEDA_B_OFF};
		if(dmx_buffer_pwm[3] > 0){LEDB_R_ON}else{LEDB_R_OFF};
		if(dmx_buffer_pwm[4] > 0){LEDB_G_ON}else{LEDB_G_OFF};
		if(dmx_buffer_pwm[5] > 0){LEDB_B_ON}else{LEDB_B_OFF};
	}
	pwm_counter++;
	if(pwm_counter == dmx_buffer_pwm[0]) LEDA_R_OFF;
	if(pwm_counter == dmx_buffer_pwm[1]) LEDA_G_OFF;
	if(pwm_counter == dmx_buffer_pwm[2]) LEDA_B_OFF;
	if(pwm_counter == dmx_buffer_pwm[3]) LEDB_R_OFF;
	if(pwm_counter == dmx_buffer_pwm[4]) LEDB_G_OFF;
	if(pwm_counter == dmx_buffer_pwm[5]) LEDB_B_OFF;
	return;
}

//############################################################################
//Hauptprogramm DMX FUNK BERTRAGUNG
int main(void)
//############################################################################
{	
	unsigned int tmp;
	unsigned char packet_cnt = 0;
	DDRD |= (1<<PD2); //Direction RS485 RX/TX
	PORTD |= (1<<PD2); //TX RS485
	
	PORTC = 0xff; //PORT Pullup

	LEDA_R_PIN_OUT;
	LEDA_G_PIN_OUT;
	LEDA_B_PIN_OUT;
	LEDB_R_PIN_OUT;
	LEDB_G_PIN_OUT;
	LEDB_B_PIN_OUT;
	
	for(unsigned long b = 0;b<100000;b++){;asm("nop");};

	rf22_init();
	rf22_setfreq(RF22FREQ(433.9));
	rf22_rxmode();	
		
	//Timer2 Timecounter fr DMX Ausfall Funk
	TCCR2A |= (1<<WGM21);
	TCCR2B |= (1<<CS20);
	TIMSK2 |= (1<<OCIE2A);	
	OCR2A = 255; //Tick Time
		
	//Init usart DMX-BUS
	UBRR0   = (F_CPU / (DMX_BAUD * 16L) - 1);
	DDRD |= (1<<PD1); //Output TXD Pin ATmega88
	UCSR0B|=(1<<TXEN0)|(1<<TXCIE0); // TXEN0 Transmitter enable / TXCIE0 TX complete interrupt enable 
	UCSR0C|=(1<<USBS0); //USBS0 2 Stop bits	
	
	sei();//Globale Interrupts Enable
	UDR0 = 0;//Start DMX
		
	while(1)
	{	
		//wurde Packet empfangen?
		if(!(PIND&(1<<3)))
		{
			tmp = rf22_getpacket(dmx_tmp_buffer); //Packetcounter und 32DMX Bytes
					
			//wurden alle Bytes empfangen?
			if(tmp == 33)
			{
				//512 Kanle werden zu je 32Bytes a 16 Packete empfangen
				packet_cnt = dmx_tmp_buffer[0];
				memcpy((unsigned char*)&dmx_buffer[(packet_cnt*32)],(unsigned char *)&dmx_tmp_buffer[1],32);
			}
		}
		
		tmp = 1 + ((~PINC) & 0x3F)*6;
		dmx_buffer_pwm[0] = dmx_buffer[tmp+5];
		dmx_buffer_pwm[1] = dmx_buffer[tmp+3];
		dmx_buffer_pwm[2] = dmx_buffer[tmp+4];
		
		dmx_buffer_pwm[3] = dmx_buffer[tmp];
		dmx_buffer_pwm[4] = dmx_buffer[tmp+2];
		dmx_buffer_pwm[5] = dmx_buffer[tmp+1];
	}
}